@@ -216,11 +216,11 @@ def inject_planet(frames, centers, inputflux, astr_hdrs, radius, pa, fwhm=3.5, t
             if xmax >= nx:
                 ypsf = ypsf[:,:-(xmax-nx + 1)]
                 xpsf = xpsf[:,:-(xmax-nx + 1)]
-                xmax = nx
+                xmax = nx - 1
             if ymax >= ny:
                 ypsf = ypsf[:-(ymax-ny + 1),:]
                 xpsf = xpsf[:-(ymax-ny + 1),:]
-                ymax = ny
+                ymax = ny - 1
 
             psf = ndimage.map_coordinates(inputpsf, [ypsf, xpsf], mode='constant', cval=0.0)
             # if specified, make field dependent PSF correction
@@ -244,7 +244,7 @@ def inject_planet(frames, centers, inputflux, astr_hdrs, radius, pa, fwhm=3.5, t
 
 def generate_dataset_with_fakes(dataset, fake_position_dict, fake_flux_dict, spectrum = None, PSF_cube = None, PSF_cube_wvs = None,
                                 star_type = None, mute = False, SpT_file_csv = None, real_planets_pos = None, sep_skip_real_pl = None,
-                                pa_skip_real_pl = None,dn_per_contrast=None):
+                                pa_skip_real_pl = None,dn_per_contrast=None,star_spec = None):
     '''
     Generate spectral datacubes with fake planets.
     It will do a copy of the cubes read in GPIData after having injected fake planets in them.
@@ -303,6 +303,8 @@ def generate_dataset_with_fakes(dataset, fake_position_dict, fake_flux_dict, spe
         pa_skip_real_pl: Limit in position angle  of how close a fake can be injected of a known GOI.
         dn_per_contrast: array of the same size as spectrum giving the conversion between the peak flux of a planet in
                         data number and its contrast.
+        star_spec: 1D array stellar spectrum sampling dataset.wvs. Otherwise uses a pickles spectrum in which the
+                   temperature in interpolated from the spectral type.
 
     '''
 
@@ -346,10 +348,13 @@ def generate_dataset_with_fakes(dataset, fake_position_dict, fake_flux_dict, spe
     if np.size(np.unique(dataset.wvs)) == 1:
         star_sp = np.ones(dn_per_contrast.shape)
     else:
-        if star_type is None:
-            star_type = spec.get_specType(star_name,SpT_file_csv)
-        # Interpolate a spectrum of the star based on its spectral type/temperature
-        wv,star_sp = spec.get_star_spectrum(dataset.wvs,star_type)
+        if star_spec is None:
+            if star_type is None:
+                star_type = spec.get_specType(star_name,SpT_file_csv)
+            # Interpolate a spectrum of the star based on its spectral type/temperature
+            wv,star_sp = spec.get_star_spectrum(dataset.wvs,star_type)
+        else:
+            star_sp = star_spec
 
     # Define the output Foldername
     if isinstance(spectrum, str):
@@ -681,7 +686,7 @@ def gaussfit2d(frame, xguess, yguess, searchrad=5, guessfwhm=3, guesspeak=1, ref
 
 
     #now calculate flux
-    xfitbox, yfitbox = np.meshgrid(np.arange(0,2* searchrad+1, 1.0)-xfit, np.arange(0, 2*searchrad+1, 1.0)-yfit)
+    xfitbox, yfitbox = np.meshgrid(np.arange(0,fitbox.shape[1], 1.0)-xfit, np.arange(0, fitbox.shape[0], 1.0)-yfit)
     #correlate data with a gaussian to get flux
     sigma = fwhm/(2*np.sqrt(2*np.log(2)))
     ## attempt to calculate sigma using moments
