@@ -18,7 +18,8 @@ class FMPlanetPSF(NoFM):
     """
     Forward models the PSF of the planet through KLIP. Returns the forward modelled planet PSF
     """
-    def __init__(self, inputs_shape, numbasis, sep, pa, dflux, input_psfs, input_wvs, flux_conversion=None, spectrallib=None, spectrallib_units="flux", star_spt=None, refine_fit=False):
+    def __init__(self, inputs_shape, numbasis, sep, pa, dflux, input_psfs, input_wvs, flux_conversion=None, spectrallib=None, 
+                 spectrallib_units="flux", star_spt=None, refine_fit=False, field_dependent_correction=None, input_psfs_pas=None):
         """
         Defining the planet to characterizae
 
@@ -29,6 +30,7 @@ class FMPlanetPSF(NoFM):
             pa: position angle of the planet
             dflux: guess for contrast of planet averaged across band w.r.t star
             input_psfs: the psf of the image. A numpy array with shape (wv, y, x)
+                        shape of (N_cubes, wvs, y, x) also acceptable to have PSFs change in time. Need to supply input_psfs_pas in that case.
             input_wvs: the wavelegnths that correspond to the input psfs
                 (doesn't need to be tiled to match the dimension of the input data of the instrument class)
             flux_conversion: an array of length N to convert from contrast to DN for each frame. Units of DN/contrast. 
@@ -37,6 +39,8 @@ class FMPlanetPSF(NoFM):
             spectrallib_units: can be either "flux"" or "contrast". Flux units requires dividing by the flux of the star to get contrast 
             star_spt: star spectral type, if None default to some random one
             refine_fit: (NOT implemented) refine the separation and pa supplied
+            field_dependent_correction: function to implement field dependent correction. See BKA tutorial.
+            input_psfs_pas: the parangs corresponding to the input PSFs (optional). Array of size N_cubes
         """
         # allocate super class
         super(FMPlanetPSF, self).__init__(inputs_shape, numbasis)
@@ -46,6 +50,7 @@ class FMPlanetPSF(NoFM):
         self.sep = sep
         self.pa = pa
         self.dflux = dflux
+        self.field_dependent_correction = field_dependent_correction
 
         if spectrallib_units.lower() != "flux" and spectrallib_units.lower() != "contrast":
             raise ValueError("spectrallib_units needs to be either 'flux' or 'contrast', not {0}".format(spectrallib_units))
@@ -79,34 +84,50 @@ class FMPlanetPSF(NoFM):
         self.psf_centx_notscaled = {}
         self.psf_centy_notscaled = {}
 
-        numwv,ny_psf,nx_psf =  self.input_psfs.shape
-        x_psf_grid, y_psf_grid = np.meshgrid(np.arange(nx_psf * 1.) - nx_psf//2, np.arange(ny_psf * 1.) - ny_psf//2)
-        psfs_func_list = []
-        for wv_index in range(numwv):
-            model_psf = self.input_psfs[wv_index, :, :] #* self.flux_conversion * self.spectrallib[0][wv_index] * self.dflux
-            #psfs_interp_model_list.append(interpolate.bisplrep(x_psf_grid,y_psf_grid,model_psf))
-            #psfs_interp_model_list.append(interpolate.SmoothBivariateSpline(x_psf_grid.ravel(),y_psf_grid.ravel(),model_psf.ravel()))
-            psfs_func_list.append(interpolate.LSQBivariateSpline(x_psf_grid.ravel(),y_psf_grid.ravel(),model_psf.ravel(),x_psf_grid[0,0:nx_psf-1]+0.5,y_psf_grid[0:ny_psf-1,0]+0.5))
-            #psfs_interp_model_list.append(interpolate.interp2d(x_psf_grid,y_psf_grid,model_psf,kind="cubic",bounds_error=False,fill_value=0.0))
-            #psfs_interp_model_list.append(interpolate.Rbf(x_psf_grid,y_psf_grid,model_psf,function="gaussian"))
+        if len(self.input_psfs.shape) == 3:
+            numwv,ny_psf,nx_psf =  self.input_psfs.shape
+            x_psf_grid, y_psf_grid = np.meshgrid(np.arange(nx_psf * 1.) - nx_psf//2, np.arange(ny_psf * 1.) - ny_psf//2)
+            psfs_func_list = []
+            for wv_index in range(numwv):
+                model_psf = self.input_psfs[wv_index, :, :] #* self.flux_conversion * self.spectrallib[0][wv_index] * self.dflux
+                #psfs_interp_model_list.append(interpolate.bisplrep(x_psf_grid,y_psf_grid,model_psf))
+                #psfs_interp_model_list.append(interpolate.SmoothBivariateSpline(x_psf_grid.ravel(),y_psf_grid.ravel(),model_psf.ravel()))
+                psfs_func_list.append(interpolate.LSQBivariateSpline(x_psf_grid.ravel(),y_psf_grid.ravel(),model_psf.ravel(),x_psf_grid[0,0:nx_psf-1]+0.5,y_psf_grid[0:ny_psf-1,0]+0.5))
+                #psfs_interp_model_list.append(interpolate.interp2d(x_psf_grid,y_psf_grid,model_psf,kind="cubic",bounds_error=False,fill_value=0.0))
+                #psfs_interp_model_list.append(interpolate.Rbf(x_psf_grid,y_psf_grid,model_psf,function="gaussian"))
 
-            if 0:
-                import matplotlib.pylab as plt
-                #print(x_psf_grid.shape)
-                #print(psfs_interp_model_list[wv_index](x_psf_grid.ravel(),y_psf_grid.ravel()).shape)
-                a = psfs_func_list[wv_index](x_psf_grid[0,:],y_psf_grid[:,0]).transpose()
-                plt.figure()
-                plt.subplot(1,3,1)
-                plt.imshow(a,interpolation="nearest")
-                plt.colorbar()
-                ##plt.imshow(psfs_interp_model_list[wv_index](np.linspace(-10,10,500),np.linspace(-10,10,500)),interpolation="nearest")
-                plt.subplot(1,3,2)
-                plt.imshow(self.input_psfs[wv_index, :, :],interpolation="nearest")
-                plt.colorbar()
-                plt.subplot(1,3,3)
-                plt.imshow(abs(self.input_psfs[wv_index, :, :]-a),interpolation="nearest")
-                plt.colorbar()
-                plt.show()
+                if 0:
+                    import matplotlib.pylab as plt
+                    #print(x_psf_grid.shape)
+                    #print(psfs_interp_model_list[wv_index](x_psf_grid.ravel(),y_psf_grid.ravel()).shape)
+                    a = psfs_func_list[wv_index](x_psf_grid[0,:],y_psf_grid[:,0]).transpose()
+                    plt.figure()
+                    plt.subplot(1,3,1)
+                    plt.imshow(a,interpolation="nearest")
+                    plt.colorbar()
+                    ##plt.imshow(psfs_interp_model_list[wv_index](np.linspace(-10,10,500),np.linspace(-10,10,500)),interpolation="nearest")
+                    plt.subplot(1,3,2)
+                    plt.imshow(self.input_psfs[wv_index, :, :],interpolation="nearest")
+                    plt.colorbar()
+                    plt.subplot(1,3,3)
+                    plt.imshow(abs(self.input_psfs[wv_index, :, :]-a),interpolation="nearest")
+                    plt.colorbar()
+                    plt.show()
+            self.psfs_in_time = False
+        else:
+            self.input_psfs_pas = input_psfs_pas
+            n_cubes,numwv,ny_psf,nx_psf =  self.input_psfs.shape
+            x_psf_grid, y_psf_grid = np.meshgrid(np.arange(nx_psf * 1.) - nx_psf//2, np.arange(ny_psf * 1.) - ny_psf//2)
+
+            psfs_func_list = []
+            for pa_index in range(n_cubes):
+                psfs_func_list_perpa = []
+                for wv_index in range(numwv):
+                    model_psf = self.input_psfs[pa_index, wv_index, :, :] #* self.flux_conversion * self.spectrallib[0][wv_index] * self.dflux
+                    psfs_func_list_perpa.append(interpolate.LSQBivariateSpline(x_psf_grid.ravel(),y_psf_grid.ravel(),model_psf.ravel(),x_psf_grid[0,0:nx_psf-1]+0.5,y_psf_grid[0:ny_psf-1,0]+0.5))
+                psfs_func_list.append(psfs_func_list_perpa)
+    
+            self.psfs_in_time = True
 
         self.psfs_func_list = psfs_func_list
 
@@ -170,31 +191,30 @@ class FMPlanetPSF(NoFM):
         Return:
             models: array of size (N, p) where p is the number of pixels in the segment
         """
-        # create some parameters for a blank canvas to draw psfs on
-        nx = input_img_shape[1]
-        ny = input_img_shape[0]
+        # create a blank canvas the same size as the input image centered at the psf center
+        nx = input_img_shape[1] # x dimension of input image
+        ny = input_img_shape[0] # y dimension of input image
         x_grid, y_grid = np.meshgrid(np.arange(nx * 1.)-ref_center[0], np.arange(ny * 1.)-ref_center[1])
 
-        numwv, ny_psf, nx_psf =  self.input_psfs.shape
+        # retrieve wavelength, x and y dimensions of input psf
+        numwv, ny_psf, nx_psf =  self.input_psfs.shape[-3:]
 
-        # create bounds for PSF stamp size
-        row_m = int(np.floor(ny_psf/2.0))    # row_minus
-        row_p = int(np.ceil(ny_psf/2.0))     # row_plus
-        col_m = int(np.floor(nx_psf/2.0))   # col_minus
-        col_p = int(np.ceil(nx_psf/2.0))    # col_plus
+        # create stamp for instrumental psf
+        psf_lower = int(np.floor(ny_psf/2.0))    # lower bound
+        psf_upper = int(np.ceil(ny_psf/2.0))     # upper bound
+        psf_left = int(np.floor(nx_psf/2.0))   # left bound
+        psf_right = int(np.ceil(nx_psf/2.0))    # right bound
 
-        # a blank img array of write model PSFs into
+        # a blank img array to write model PSFs into
         whiteboard = np.zeros((ny,nx))
         if debug:
             canvases = []
         models = []
-        #print(self.input_psfs.shape)
+
         for pa, wv in zip(pas, wvs):
-            #print(self.pa,self.sep)
-            #print(pa,wv)
+
             # grab PSF given wavelength
             wv_index = spec.find_nearest(self.input_psfs_wvs,wv)[1]
-            #model_psf = self.input_psfs[wv_index[0], :, :] #* self.flux_conversion * self.spectrallib[0][wv_index] * self.dflux
 
             # find center of psf
             # to reduce calculation of sin and cos, see if it has already been calculated before
@@ -210,29 +230,59 @@ class FMPlanetPSF(NoFM):
 
             # create a coordinate system for the image that is with respect to the model PSF
             # round to nearest pixel and add offset for center
-            l = int(round(psf_centx + ref_center[0]))
-            k = int(round(psf_centy + ref_center[1]))
-            # recenter coordinate system about the location of the planet
-            x_vec_stamp_centered = np.copy(x_grid[0, (l-col_m):(l+col_p)]) - psf_centx
-            y_vec_stamp_centered = np.copy(y_grid[(k-row_m):(k+row_p), 0]) - psf_centy
+            planet_centx = int(round(psf_centx + ref_center[0]))
+            planet_centy = int(round(psf_centy + ref_center[1]))
+            
+            # recenter stamp coordinate system about the location of the planet
+            stamp_left = planet_centx-psf_left
+            stamp_right = planet_centx+psf_right
+            stamp_lower = planet_centy-psf_lower
+            stamp_upper = planet_centy+psf_upper
+
+            # Save the stamp length and width as variables for slicing the image
+            stamp_len = slice(stamp_lower, stamp_upper)
+            stamp_width = slice(stamp_left,stamp_right)
+
+            x_vec_stamp_centered = np.copy(x_grid[0, stamp_width]) - psf_centx
+            y_vec_stamp_centered = np.copy(y_grid[stamp_len, 0]) - psf_centy
             # rescale to account for the align and scaling of the refernce PSFs
             # e.g. for longer wvs, the PSF has shrunk, so we need to shrink the coordinate system
             x_vec_stamp_centered /= (ref_wv/wv)
             y_vec_stamp_centered /= (ref_wv/wv)
 
-            # use intepolation spline to generate a model PSF and write to temp img
-            whiteboard[(k-row_m):(k+row_p), (l-col_m):(l+col_p)] = \
-                    self.psfs_func_list[wv_index](x_vec_stamp_centered,y_vec_stamp_centered).transpose()
 
+            # use intepolation spline to generate a model PSF of planet and write to temp img
+            if not self.psfs_in_time:
+                # just grab the right wavelength
+                psf_func = self.psfs_func_list[wv_index]
+            else:
+                pa_index = spec.find_nearest(self.input_psfs_pas, pa)[1]
+                psf_func = self.psfs_func_list[pa_index][wv_index]
+            whiteboard[stamp_len, stamp_width] = \
+                    psf_func(x_vec_stamp_centered,y_vec_stamp_centered).transpose()
+            
+            # if specified, make field dependent PSF correction
+            if self.field_dependent_correction is not None:
+
+                # find distance from center in x and y dimensions
+                dx = x_grid[stamp_len, stamp_width]
+                dy = y_grid[stamp_len, stamp_width]
+                whiteboard[stamp_len, stamp_width] = \
+                        self.field_dependent_correction(whiteboard[stamp_len, stamp_width], dx, dy)
+    
             # write model img to output (segment is collapsed in x/y so need to reshape)
             whiteboard.shape = [input_img_shape[0] * input_img_shape[1]]
             segment_with_model = copy(whiteboard[section_ind])
             whiteboard.shape = [input_img_shape[0],input_img_shape[1]]
 
+           
+
             models.append(segment_with_model)
 
             # clean whiteboard
-            whiteboard[(k-row_m):(k+row_p), (l-col_m):(l+col_p)] = 0
+            whiteboard[stamp_len, stamp_width] = 0
+
+        
 
 
         return np.array(models)
