@@ -7,6 +7,7 @@ import astropy.wcs as wcs
 from astropy.modeling import models, fitting
 import numpy as np
 import scipy.ndimage as ndimage
+import pyklip
 from pyklip.parallelized import high_pass_filter_imgs
 import scipy.stats
 import multiprocessing as mp
@@ -355,7 +356,7 @@ class MagAOData(object):
         # the universal_newline argument is just so python3 returns a string instead of bytes
         # this will probably come to bite me later
         try:
-            pyklipver = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'], cwd=pykliproot, universal_newlines=True).strip()
+            pyklipver = pyklip.__version__
         except:
             pyklipver = "unknown"
         hdulist[0].header['PSFSUB'] = "pyKLIP"
