@@ -29,6 +29,7 @@ else:
     from pyklip.instruments.Instrument import Data
     from pyklip.instruments.utils.nair import nMathar
 
+import pyklip
 from pyklip.instruments.P1640_support import P1640spots
 from pyklip.instruments.P1640_support import P1640utils
 # from pyklip.instruments.P1640_support import P1640_cube_checker
@@ -482,7 +483,7 @@ class P1640Data(Data):
         # the universal_newline argument is just so python3 returns a string instead of bytes
         # this will probably come to bite me later
         try:
-            pyklipver = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'], cwd=pykliproot, universal_newlines=True).strip()
+            pyklipver = pyklip.__version__
         except:
             pyklipver = "unknown"
         hdulist[0].header['PSFSUB'] = ("pyKLIP", "PSF Subtraction Algo")
