@@ -4,6 +4,7 @@ import scipy.linalg as la
 import scipy.ndimage as ndimage
 import scipy.interpolate as sinterp
 from scipy.stats import t
+import warnings
 
 def make_polar_coordinates(x, y, center=[0,0]):
     '''
@@ -32,40 +33,52 @@ def collapse_data(data, pixel_weights=None, axis=1, collapse_method='mean'):
         data: (multi-dimension)arrays of 2D images or 3D cubes.
         pixel_weights: ones if collapse method is not weighted collapse
         axis: axis index along which to collapse
-        collapse_method: currently support 'median', 'mean', 'weighted-mean', 'trimmed-mean'
+        collapse_method: currently support 'median', 'mean', 'weighted-mean', 'trimmed-mean', 'weighted-median'
 
     Returns:
         Collapsed data
     """
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=RuntimeWarning)
 
-    if collapse_method.lower() == 'median':
+        if collapse_method.lower() == 'median':
 
-        return np.nanmedian(data, axis=axis)
+            return np.nanmedian(data, axis=axis)
 
-    elif collapse_method.lower() == 'mean':
+        elif collapse_method.lower() == 'mean':
 
-        return np.nanmean(data, axis=axis)
+            return np.nanmean(data, axis=axis)
 
-    elif ('weighted' in collapse_method.lower()) and ('mean' in collapse_method.lower()):
-        if pixel_weights is None:
-            pixel_weights = np.ones(data.shape)
-        collapsed_data = np.nanmean(pixel_weights * data, axis=axis)
-        collapsed_data /= np.nanmean(pixel_weights, axis=axis)
 
-        return collapsed_data
 
-    elif ('trimmed' in collapse_method.lower()) and ('mean' in collapse_method.lower()):
-        collapsed = np.sort(data, axis=axis)
-        collapsed = np.take(collapsed, range(2, collapsed.shape[axis] - 2), axis=axis)
-        collapsed = np.nanmean(collapsed, axis=axis)
+        elif ('weighted' in collapse_method.lower()) and ('mean' in collapse_method.lower()):
+            if pixel_weights is None:
+                pixel_weights = np.ones(data.shape)
+            collapsed_data = np.nanmean(pixel_weights * data, axis=axis)
+            collapsed_data /= np.nanmean(pixel_weights, axis=axis)
 
-        return collapsed
+            return collapsed_data
 
-    else:
-        # default to mean collapse if input does not match any supported pattern
-        print('{} collapse not supported, using mean collapse instead...'.format(collapse_method))
+        elif ('trimmed' in collapse_method.lower()) and ('mean' in collapse_method.lower()):
+            collapsed = np.sort(data, axis=axis)
+            collapsed = np.take(collapsed, range(2, collapsed.shape[axis] - 2), axis=axis)
+            collapsed = np.nanmean(collapsed, axis=axis)
 
-        return np.nanmean(data, axis=axis)
+            return collapsed
+
+        elif ('weighted' in collapse_method.lower()) and ('median' in collapse_method.lower()):
+            if pixel_weights is None:
+                pixel_weights = np.ones(data.shape)
+            collapsed_data = np.nanmedian(pixel_weights * data, axis=axis)
+            collapsed_data /= np.nanmedian(pixel_weights, axis=axis)
+
+            return collapsed_data
+
+        else:
+            # default to mean collapse if input does not match any supported pattern
+            print('{} collapse not supported, using mean collapse instead...'.format(collapse_method))
+
+            return np.nanmean(data, axis=axis)
 
 
 def klip_math(sci, ref_psfs, numbasis, covar_psfs=None, return_basis=False, return_basis_and_eig=False):
@@ -304,10 +317,10 @@ def nan_map_coordinates_2d(img, yp, xp, mc_kwargs=None):
         mc_kwargs["cval"] = np.nan
 
     # check all four pixels around each pixel and see whether they are nans
-    xp_floor = np.clip(np.floor(xp).astype(int), 0, xp.shape[1]-1)
-    xp_ceil = np.clip(np.ceil(xp).astype(int), 0, xp.shape[1]-1)
-    yp_floor = np.clip(np.floor(yp).astype(int), 0, yp.shape[0]-1)
-    yp_ceil = np.clip(np.ceil(yp).astype(int), 0, yp.shape[0]-1)
+    xp_floor = np.clip(np.floor(xp).astype(int), 0, img.shape[1]-1)
+    xp_ceil = np.clip(np.ceil(xp).astype(int), 0, img.shape[1]-1)
+    yp_floor = np.clip(np.floor(yp).astype(int), 0, img.shape[0]-1)
+    yp_ceil = np.clip(np.ceil(yp).astype(int), 0, img.shape[0]-1)
     rotnans = np.where(np.isnan(img[yp_floor.ravel(), xp_floor.ravel()]) | 
                        np.isnan(img[yp_floor.ravel(), xp_ceil.ravel()]) |
                        np.isnan(img[yp_ceil.ravel(), xp_floor.ravel()]) |
@@ -470,7 +483,8 @@ def meas_contrast(dat, iwa, owa, resolution, center=None, low_pass_filter=True):
         dat: 2D image - already flux calibrated
         iwa: inner working angle
         owa: outer working angle
-        resolution: size of resolution element in pixels (FWHM or lambda/D)
+        resolution: size of noise resolution element in pixels (for speckle noise ~ FWHM or lambda/D)
+                    but it can be 1 pixel if limited by pixel-to-pixel noise. 
         center: location of star (x,y). If None, defaults the image size // 2.
         low_pass_filter: if True, run a low pass filter.
                          Can also be a float which specifices the width of the Gaussian filter (sigma).
@@ -558,8 +572,8 @@ def nan_gaussian_filter(img, sigma):
     filtered /= filter_norm
     filtered[nan_locs] = np.nan
 
-    # for some reason, the fitlered image peak pixel fluxes get decreased by 2
-    filtered *= 2
+    # (2020-09-10: checking the gain for each image, it seems this is no longer a problem, next line is commented out)
+    #filtered *= 2
 
     return filtered
 
