@@ -23,9 +23,12 @@ try:
 except ImportError:
     mkl_exists = False
 
-# can turn off for debugging purposes
+# Turns parallelism off for debugging purposes
 global parallel
-parallel = True
+debug = False
+
+if debug is True:
+    print("You turned on debug mode, so parallelism is off. Code may be slower.")
 
 
 def _tpool_init(original_imgs, original_imgs_shape, aligned_imgs, aligned_imgs_shape, output_imgs, output_imgs_shape,
@@ -104,7 +107,7 @@ def _save_spectral_cubes(dataset, pixel_weights, time_collapse, numbasis, flux_c
 
 
 def _save_wv_collapsed_images(dataset, pixel_weights, numbasis, time_collapse, wv_collapse, num_wvs,
-                              spectrum, spectra_template, flux_cal, outputdirpath, fileprefix):
+                              spectrum, spectra_template, flux_cal, outputdirpath, fileprefix, verbose = True):
     """
     Saves KLmode cube, shape (b, y, x), each slice is a 2D image collapsed along both time and
     wavelength dimension for a specific numbasis
@@ -113,7 +116,7 @@ def _save_wv_collapsed_images(dataset, pixel_weights, numbasis, time_collapse, w
         dataset: an instance of CHARISData
         pixel_weights: pixel weights for dataset.output, same shape as dataset.output
         numbasis: number of KL basis vectors to use (can be a scalar or list like). Length of b
-        time_collapse: method for time collapse. Support: 'median', 'mean', 'weighted-mean'
+        time_collapse: method for time collapse. Support: 'median', 'mean', 'weighted-mean', 'weighted-median'
         wv_collapse: method for wavelength collapse. Supports: 'median', 'mean', 'weighted-mean', 'trimmed-mean'
         num_wvs: number of wavelength slices
         spectrum: refer to klip_dataset() docstring
@@ -125,8 +128,8 @@ def _save_wv_collapsed_images(dataset, pixel_weights, numbasis, time_collapse, w
     Returns:
         saves wavelength collapsed images to output
     """
-
-    print('wavelength collapsing reduced data of shape (b, N, wv, y, x):{}'.format(dataset.output.shape))
+    if verbose is True:
+        print('wavelength collapsing reduced data of shape (b, N, wv, y, x):{}'.format(dataset.output.shape))
 
     spectral_cubes = klip.collapse_data(dataset.output, pixel_weights, axis=1, collapse_method=time_collapse) # spectral_cubes shape (b, wv, y, x)
 
@@ -283,7 +286,8 @@ def _klip_section(img_num, parang, wavelength, wv_index, numbasis, radstart, rad
     #grab the pixel location of the section we are going to anaylze
     section_ind = np.where((r >= radstart) & (r < radend) & (phi >= phistart) & (phi < phiend))
     if np.size(section_ind) == 0:
-        print("section is empty, skipping...")
+        if verbose is True:
+            print("section is empty, skipping...")
         return False
 
     #grab the files suitable for reference PSF
@@ -295,7 +299,8 @@ def _klip_section(img_num, parang, wavelength, wv_index, numbasis, radstart, rad
     moves = klip.estimate_movement(avg_rad, parang, pa_imgs, wavelength, wvs_imgs)
     file_ind = np.where(moves >= minmove)
     if np.size(file_ind[0]) < 1:
-        print("less than 1 reference PSFs available for minmove={0}, skipping...".format(minmove))
+        if verbose is True:
+            print("less than 1 reference PSFs available for minmove={0}, skipping...".format(minmove))
         return False
 
     #load aligned images and make reference PSFs
@@ -393,7 +398,8 @@ def _klip_section_multifile(scidata_indices, wavelength, wv_index, numbasis, max
     #grab the pixel location of the section we are going to anaylze
     section_ind = np.where((r >= radstart) & (r < radend) & (phi >= phistart) & (phi < phiend))
     if np.size(section_ind) <= 1:
-        print("section is too small ({0} pixels), skipping...".format(np.size(section_ind)))
+        if verbose is True:
+            print("section is too small ({0} pixels), skipping...".format(np.size(section_ind)))
         return False
 
     #export some of klip.klip_math functions to here to minimize computation repeats
@@ -446,6 +452,7 @@ def _klip_section_multifile(scidata_indices, wavelength, wv_index, numbasis, max
     #we have to correct for that in the klip.klip_math routine when consturcting the KL
     #vectors since that's not part of the equation in the KLIP paper
     covar_psfs = np.cov(ref_psfs_mean_sub)
+        
     if ref_psfs_mean_sub.shape[0] == 1:
         # EDGE CASE: if there's only 1 image, we need to reshape to covariance matrix into a 2D matrix
         covar_psfs = covar_psfs.reshape((1,1))
@@ -475,6 +482,10 @@ def _klip_section_multifile(scidata_indices, wavelength, wv_index, numbasis, max
             ref_psfs_smoothed.append(smoothed_section)
     
         corr_psfs = np.corrcoef(ref_psfs_smoothed)
+        if ref_psfs_mean_sub.shape[0] == 1:
+            # EDGE CASE: if there's only 1 image, we need to reshape the correlation matrix into a 2D matrix
+            corr_psfs = corr_psfs.reshape((1,1))
+            
         # smoothing could have caused some ref images to have all 0s
         # which would give them correlation matrix entries of NaN
         # 0 them out for now.
@@ -513,7 +524,7 @@ def _klip_section_multifile_perfile(img_num, section_ind, ref_psfs, covar,  corr
                                     psflib_good=None, psflib_corr=None,
                                     spectrum=None, lite=False, dtype=None, algo='klip'):
     """
-    Imitates the rest of _klip_section for the multifile code. Does the rest of the PSF reference selection
+    Imitates the rest of _klip_section for the multifile code. Does the rest of the PSF reference selection and runs KLIP.
 
     Args:
         img_num: file index for the science image to process
@@ -582,7 +593,8 @@ def _klip_section_multifile_perfile(img_num, section_ind, ref_psfs, covar,  corr
     ref2rm = np.where(np.nansum(np.isfinite(ref_psfs[good_file_ind[0], :]),axis=1) < 5)[0]
     good_file_ind = (np.delete(good_file_ind[0],ref2rm),)
     if (np.size(good_file_ind[0]) < 1) and (not include_rdi):
-        print("less than 1 reference PSFs available for minmove={0}, skipping...".format(minmove))
+        if verbose is True:
+            print("less than 1 reference PSFs available for minmove={0}, skipping...".format(minmove))
         return False
     # pick out a subarray. Have to play around with indicies to get the right shape to index the matrix
     covar_files = covar[good_file_ind[0].reshape(np.size(good_file_ind), 1), good_file_ind[0]]
@@ -663,15 +675,25 @@ def _klip_section_multifile_perfile(img_num, section_ind, ref_psfs, covar,  corr
 
         if include_rdi:
             rdi_psfs_selected = psf_library[psflib_good][:, section_ind[0]]
-
+    
     # add PSF library to reference psf list and covariance matrix if needed
     if include_rdi:
+
+        #subctract the mean and remove the Nans from the RDI PSFs before measuring the covariance.
+        # this was already done in _klip_section_multifile for the other PSFs)
+        rdi_psfs_selected = rdi_psfs_selected - np.nanmean(rdi_psfs_selected, axis=1)[:, None]
+        rdi_psfs_selected[np.where(np.isnan(rdi_psfs_selected))] = 0
+
         # compute covariances. I could just grab these from ~20 lines above, but too lazy
         rdi_covar = np.cov(rdi_psfs_selected) # N_rdi_sel x N_rdi_sel
+        # EDGE CASE: if there's only 1 image, we need to reshape to covariance matrix into a 2D matrix
+        if not rdi_covar.shape:
+            rdi_covar = rdi_covar.reshape([1,1])
+        
         # compute cross term
         # cross term has shape N_dataset_ref x N_rdi_selected
         covar_ref_x_rdi = np.dot((ref_psfs_selected - np.nanmean(ref_psfs_selected, axis=1)[:,None]),
-                                 (rdi_psfs_selected - np.mean(rdi_psfs_selected, axis=1)[:,None]).T) / (numpix - 1)
+                                 (rdi_psfs_selected - np.nanmean(rdi_psfs_selected, axis=1)[:,None]).T) / (numpix - 1)
         # piece together covariance matrix. It should looke like
         # [ cov_ref, cov_ref_x_rdi ]
         # [ cov_rdi_x_ref, cov_rdi ]
@@ -685,7 +707,7 @@ def _klip_section_multifile_perfile(img_num, section_ind, ref_psfs, covar,  corr
         ref_psfs_selected = np.append(ref_psfs_selected, rdi_psfs_selected, axis=0)
 
 
-
+    # output_images has shape (N, y*x, b) and not (N, y, x, b) as normal
     output_imgs = _arraytonumpy(output, (output_shape[0], output_shape[1]*output_shape[2], output_shape[3]),dtype=dtype)
 
     # run KLIP
@@ -703,7 +725,7 @@ def _klip_section_multifile_perfile(img_num, section_ind, ref_psfs, covar,  corr
         print(err.args)
         return False
 
-    # write to output
+    # write to output 
     output_imgs[img_num, section_ind[0], :] = klipped
 
     return True
@@ -976,7 +998,7 @@ def klip_parallelized_lite(imgs, centers, parangs, wvs, filenums, IWA, OWA=None,
                               output_imgs_shape, pa_imgs, wvs_imgs, centers_imgs, filenums_imgs, None, None), maxtasksperchild=50)
 
     # SINGLE THREAD DEBUG PURPOSES ONLY
-    if not parallel:
+    if debug:
         _tpool_init(original_imgs, original_imgs_shape, recentered_imgs, recentered_imgs_shape, output_imgs,
                               output_imgs_shape, pa_imgs, wvs_imgs, centers_imgs, filenums_imgs, None, None)
 
@@ -1014,7 +1036,7 @@ def klip_parallelized_lite(imgs, centers, parangs, wvs, filenums, IWA, OWA=None,
         #perform KLIP asynchronously for each group of files of a specific wavelength and section of the image
         lite = True
 
-        if parallel:
+        if not debug:
             outputs += [tpool.apply_async(_klip_section_multifile,
                                           args=(scidata_indices, this_wv, wv_index, numbasis,
                                                 maxnumbasis,
@@ -1034,7 +1056,7 @@ def klip_parallelized_lite(imgs, centers, parangs, wvs, filenums, IWA, OWA=None,
 
         #harness the data!
         #check make sure we are completely unblocked before outputting the data
-        if parallel:
+        if not debug:
             for out in outputs:
                 out.wait()
                 if (jobs_complete + 1) % 10 == 0:
@@ -1078,9 +1100,9 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
                       numbasis=None, aligned_center=None, numthreads=None, minrot=0, maxrot=360, 
                       annuli_spacing="constant", maxnumbasis=None, corr_smooth=1,
                       spectrum=None, psf_library=None, psf_library_good=None, psf_library_corr=None,
-                      save_aligned = False, restored_aligned = None, dtype=None, algo='klip', compute_noise_cube=False):
+                      save_aligned = False, restored_aligned = None, dtype=None, algo='klip', compute_noise_cube=False, verbose = True):
     """
-    multithreaded KLIP PSF Subtraction
+    Multitprocessed KLIP PSF Subtraction
 
     Args:
         imgs: array of 2D images for ADI. Shape of array (N,y,x)
@@ -1133,7 +1155,8 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
         totalimgs = imgs.shape[0]
         maxbasis = np.min([totalimgs, 100]) #only going up to 100 KL modes by default
         numbasis = np.arange(1, maxbasis + 5, 10)
-        print("KL basis not specified. Using default.", numbasis)
+        if verbose is True:
+            print("KL basis not specified. Using default.", numbasis)
     else:
         if hasattr(numbasis, "__len__"):
             numbasis = np.array(numbasis)
@@ -1273,14 +1296,15 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
                     maxtasksperchild=50)
 
     # # SINGLE THREAD DEBUG PURPOSES ONLY
-    if not parallel:
+    if debug:
         _tpool_init(original_imgs, original_imgs_shape, recentered_imgs, recentered_imgs_shape, output_imgs,
                             output_imgs_shape, pa_imgs, wvs_imgs, centers_imgs, filenums_imgs, psf_lib, psf_lib_shape)
 
 
     if restored_aligned is None:
         #align and scale the images for each image. Use map to do this asynchronously
-        print("Begin align and scale images for each wavelength")
+        if verbose is True:
+            print("Begin align and scale images for each wavelength")
         realigned_index = tpool.imap_unordered(_align_and_scale, zip(enumerate(unique_wvs), itertools.repeat(aligned_center),itertools.repeat(dtype)))
     else:
         #align and scale the images for each image. Use map to do this asynchronously
@@ -1290,7 +1314,8 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
     outputs = []
     #as each is finishing, queue up the aligned data to be processed with KLIP
     for wv_index, wv_value in realigned_index:
-        print("Wavelength {1:.4} with index {0} has finished align and scale. Queuing for KLIP".format(wv_index, wv_value))
+        if verbose is True:
+            print("Wavelength {1:.4} with index {0} has finished align and scale. Queuing for KLIP".format(wv_index, wv_value))
 
         #pick out the science images that need PSF subtraction for this wavelength
         scidata_indices = np.where(wvs == wv_value)[0]
@@ -1305,7 +1330,7 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
         #perform KLIP asynchronously for each group of files of a specific wavelength and section of the image
         lite = False
 
-        if parallel:
+        if not debug:
             outputs += [tpool.apply_async(_klip_section_multifile, (scidata_indices, wv_value, wv_index, numbasis,
                                                                         maxnumbasis,
                                                                         radstart, radend, phistart, phiend, movement,
@@ -1328,8 +1353,9 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
 
     #harness the data!
     #check make sure we are completely unblocked before outputting the data
-    if parallel:
-        print("Total number of tasks for KLIP processing is {0}".format(tot_iter))
+    if not debug:
+        if verbose is True:
+            print("Total number of tasks for KLIP processing is {0}".format(tot_iter))
         for index, out in enumerate(outputs):
             out.wait()
             if (index + 1) % 10 == 0:
@@ -1337,7 +1363,8 @@ def klip_parallelized(imgs, centers, parangs, wvs, filenums, IWA, OWA=None, mode
 
 
     #close to pool now and make sure there's no processes still running (there shouldn't be or else that would be bad)
-    print("Closing threadpool")
+    if verbose is True:
+        print("Closing threadpool")
     tpool.close()
     tpool.join()
 
@@ -1375,7 +1402,7 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
                  numbasis=None, numthreads=None, minrot=0, calibrate_flux=False, aligned_center=None,
                  annuli_spacing="constant", maxnumbasis=None, corr_smooth=1, spectrum=None, psf_library=None, 
                  highpass=False, lite=False, save_aligned = False, restored_aligned = None, dtype=None, algo='klip',
-                 time_collapse="mean", wv_collapse='mean'):
+                 time_collapse="mean", wv_collapse='mean', verbose = True):
     """
     run klip on a dataset class outputted by an implementation of Instrument.Data
 
@@ -1412,13 +1439,14 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
 
         lite:           if True, run a low memory version of the alogirhtm
 
-        save_aligned	Save the aligned and scaled images (as well as various wcs information), True/False
-        restore_aligned The aligned and scaled images from a previous run of klip_dataset
+        save_aligned:	Save the aligned and scaled images (as well as various wcs information), True/False
+        restore_aligned: The aligned and scaled images from a previous run of klip_dataset
         				(usually restored_aligned = dataset.aligned_and_scaled)
         dtype:          data type of the arrays. Should be either ctypes.c_float(default) or ctypes.c_double
         algo (str):     algorithm to use ('klip', 'nmf', 'empca', 'none'). None will run no PSF subtraction. 
-        time_collapse:  how to collapse the data in time. Currently support: "mean", "weighted-mean", 'median'
+        time_collapse:  how to collapse the data in time. Currently support: "mean", "weighted-mean", 'median', "weighted-median"
         wv_collapse:    how to collapse the data in wavelength. Currently support: 'median', 'mean', 'trimmed-mean'
+        verbose (bool): if True, print KLIP processes. 
 
     Returns
         Saved files in the output directory
@@ -1443,7 +1471,8 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
         totalimgs = dataset.input.shape[0]
         maxbasis = np.min([totalimgs, 100]) # only going up to 100 KL modes by default
         numbasis = np.arange(1, maxbasis + 5, 10)
-        print("KL basis not specified. Using default.", numbasis)
+        if verbose is True:
+            print("KL basis not specified. Using default.", numbasis)
     else:
         if hasattr(numbasis, "__len__"):
             numbasis = np.array(numbasis)
@@ -1459,6 +1488,8 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
 
     # RDI Sanity Checks to make sure PSF Library is properly configured
     if "RDI" in mode:
+        if lite:
+            raise ValueError("RDI is currently not supported in memory lite mode.")
         if psf_library is None:
             raise ValueError("You need to pass in a psf_library if you want to run RDI")
         if psf_library.dataset is dataset:
@@ -1491,7 +1522,6 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
     if lite:
         klip_function = klip_parallelized_lite
         if (save_aligned is True) or (restored_aligned is True):
-            print('save_aligned and restored_aligned are not compatible with lite mode')
             raise ValueError('save_aligned and restored_aligned are not compatible with lite mode')
         # save_aligned = False
         # restored_aligned = None
@@ -1562,7 +1592,7 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
                     'spectrum':spectra_template, 'psf_library':master_library,
                     'psf_library_corr':rdi_corr_matrix, 'psf_library_good':rdi_good_psfs,
                     'save_aligned' : save_aligned, 'restored_aligned' : restored_aligned, 'dtype':dtype,
-                    'algo':algo, 'compute_noise_cube':weighted}
+                    'algo':algo, 'compute_noise_cube':weighted, 'verbose':verbose}
 
     #Set MLK parameters
     if mkl_exists:
@@ -1578,7 +1608,8 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
     # run KLIP
     # For SDI(+ADI)(+RDI) reductions
     if "SDI" in mode:
-        print("Beginning {0} KLIP".format(mode))
+        if verbose is True:
+            print("Beginning {0} KLIP".format(mode))
 
         # Actually run the PSF Subtraction with all the arguments
         klip_outputs = klip_function(dataset.input, dataset.centers, dataset.PAs, dataset.wvs, dataset.filenums,
@@ -1612,7 +1643,7 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
             dataset.aligned_and_scaled = []
 
         for wvindex,unique_wv in enumerate(unique_wvs):
-            if num_wvs > 1:
+            if (num_wvs > 1) and (verbose is True):
                 print("Running KLIP ADI on slice {0}/{1}: {2:.3f} um".format(wvindex+1, num_wvs, unique_wv))
             thiswv = np.where(dataset.wvs == unique_wv)
 
@@ -1682,7 +1713,8 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
         aligned_center = [int(dataset.input.shape[2]//2), int(dataset.input.shape[1]//2)]
 
     # parallelized rotate images
-    print("Derotating Images...")
+    if verbose is True:
+        print("Derotating Images...")
     rot_imgs = rotate_imgs(dataset.output, flattend_parangs, flattened_centers, numthreads=numthreads, flipx=dataset.flipx,
                            hdrs=dataset.output_wcs, new_center=aligned_center)
     # re-expand the images in num cubes/num wvs (num KLmode cutoffs, num cubes, num wvs, y, x)
@@ -1701,7 +1733,8 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
    
     # valid output path and write iamges
     outputdirpath = os.path.realpath(outputdir)
-    print("Writing Images to directory {0}".format(outputdirpath))
+    if verbose is True:
+        print("Writing Images to directory {0}".format(outputdirpath))
 
     # create weights for each pixel. If we aren't doing weighted mean, weights are just ones
     pixel_weights = 1./stddev_frames**2
@@ -1710,7 +1743,7 @@ def klip_dataset(dataset, mode='ADI+SDI', outputdir=".", fileprefix="", annuli=5
         _save_spectral_cubes(dataset, pixel_weights, time_collapse, numbasis, calibrate_flux, outputdirpath, fileprefix)
 
     _save_wv_collapsed_images(dataset, pixel_weights, numbasis, time_collapse, wv_collapse, num_wvs, spectrum,
-                              spectra_template, calibrate_flux, outputdirpath, fileprefix)
+                              spectra_template, calibrate_flux, outputdirpath, fileprefix, verbose)
 
     # Restore old setting
     if mkl_exists:
